/*
 * Copyright 1991, 1992, John F. Haugh II
 * All rights reserved.
 *
 * Permission is granted to copy and create derivative works for any
 * non-commercial purpose, provided this copyright notice is preserved
 * in all copies of source code, or included in human readable form
 * and conspicuously displayed on all copies of object code or
 * distribution media.
 */

#ifndef	lint
static	char	sccsid[] = "@(#)db_index.c	1.5	10:34:19	2/26/92";
#endif

#include <sys/types.h>
#include "is_types.h"
#include "isam.h"

extern	int	db_errno;

/*
 * db_index_name - find the name of an index starting with "field"
 *
 *	db_index_next locates an index whose first field begins with
 *	the named "field".  The shortest index is used in the hope
 *	that there is an index that is just the field "field".
 */

char *
db_index_name (dbm, file, field)
dbm_t	*dbm;
int	file;
char	*field;
{
	int	fieldnum;
	int	i;
	int	keys;
	int	length = 0;
	int	best = 0;
	is_index_t *ip;
	field_t	*fp;
	sort_key_t *kp;

	/*
	 * This file must have at least one index if any are to be
	 * found.
	 */

	if (! (ip = dbm->d_files[file].f_index))
		return (db_errno = DB_NOERR), (char *) 0;

	/*
	 * Locate the field in the file's schema.  Check real quick
	 * to see if it is an index itself.
	 */

	if ((fieldnum = db_open_field (dbm, file, field)) == -1)
		return 0;

	fp = &dbm->d_files[file].f_schema->s_fields[fieldnum];
	if (fp->f_type == T_INDEX)
		return fp->f_name;

	/*
	 * Get the list of keys and the number of keys this file has
	 * defined for it.
	 */

	keys = ip->i_key_count;
	kp = ip->i_keys;

	/*
	 * Scan through all of the keys and look at the fields in each
	 * key.  Whenever you find a key with the correct field in the
	 * first position, save it if it is better than any other key
	 * that was found so far.
	 */

	for (i = 1;i <= keys && length != 1;i++) {
		if (kp[i].k_fields[0] != fieldnum)
			continue;

		if (best == 0 || kp[i].k_field_count < length) {
			best = i;
			length = kp[i].k_field_count;
		}
	}

	/*
	 * No key was found, return NUL.  This isn't an error condition
	 * like the earlier case, so we clear db_errno.
	 */

	if (best == 0)
		return (db_errno = DB_NOERR), (char *) 0;

	/*
	 * Find the index field in the file's schema and return a pointer
	 * to its name.
	 */

	fp = &dbm->d_files[file].f_schema->s_fields[kp[best].k_field_number];
	return fp->f_name;
}
