/*
 * Copyright 1991, 1992, John F. Haugh II
 * All rights reserved.
 *
 * Permission is granted to copy and create derivative works for any
 * non-commercial purpose, provided this copyright notice is preserved
 * in all copies of source code, or included in human readable form
 * and conspicuously displayed on all copies of object code or
 * distribution media.
 */

#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#ifdef	BSD
#include <strings.h>
#else
#include <string.h>
#endif
#ifdef	MALLOC_H
#include <malloc.h>
#endif
#include "is_types.h"
#include "is_proto.h"
#include "patchlevel.h"
#include "isam.h"

#ifndef lint
static	char	sccsid[] = "@(#)dbunload.c	1.12	11:25:28	2/7/92";
#endif

#ifdef BSD
#define strchr index
#endif /* BSD */

dbm_t	*db_create();
dbm_t	*db_open();
int	db_open_file();
int	db_create_file();
int	db_create_field();
int	db_read_record();
int	db_read_view();
#ifndef	MALLOC_H
char	*malloc();
#endif

char	error_msg[BUFSIZ];

/*
 * usage - display command syntax and exit
 */

usage ()
{
	fprintf (stderr, "usage: dbunload [ -b ] [ -d delimiter ]\n");
	fprintf (stderr, "         [ -[v|V] field,... ] database file [ index ]\n");
	fprintf (stderr, "       dbunload -V\n");
	exit (1);
}

main (argc, argv)
int	argc;
char	**argv;
{
	dbm_t	*dbm;
	file_t	*file;
	field_t	*fields;
	int	filenum;
	int	fieldnum;
	char	*buf;
	char	*record;
	char	*view_fields = 0;
	char	*cp;
	short	view_list[100];
	int	view_count = 0;
	int	view_packed = 0;
	field_t	new_field;
	int	size;
	int	field;
	int	i, j;
	int	delimiter = '|';
	int	binary = 0;
	int	flag;
	int	(*db_read)();
	long	recno = 0;
	extern	int	optind;
	extern	char	*optarg;

	setbuf (stdout, 0);

	/*
	 * This is tricky - the command accepts -V for two different
	 * reasons.  The first is the "version" command which all
	 * of the commands accept.  The second is for a "packed view".
	 */

	if (argc == 2 && strcmp (argv[1], "-V") == 0) {
		fprintf (stderr, "dbunload: version %s\n", VERSION);
		exit (0);
	}

	/*
	 * Get the command line arguments.  They are -
	 *
	 *	-f - file to output records to
	 *	-b - use binary output format
	 *	-d - the option character is used as the text delimiter
	 *	-v - the selected fields are output as a view
	 *	-V - the selected fields are output as a packed view
	 */

	while ((flag = getopt (argc, argv, "f:bd:v:V:")) != EOF) {
		if (flag == 'd') {
			delimiter = *optarg;
		} else if (flag == 'b') {
			binary = 1;
		} else if (flag == 'f') {
			if (freopen (optarg, "w", stdout) == 0)
				exit (1);
		} else if (flag == 'v' || flag == 'V') {
			view_fields = optarg;
			view_packed = flag == 'V';
		} else
			usage ();
	}

	/*
	 * Make certain there are enough arguments left over.
	 */

	if (optind + 2 > argc)
		usage ();

	/*
	 * Open the named database
	 */

	if (! (dbm = db_open (argv[optind], O_RDONLY))) {
		sprintf (error_msg, "dbunload: can't open database %s",
			argv[optind]);
		db_error (error_msg);
		exit (1);
	}

	/*
	 * Open the named file
	 */

	if ((filenum = db_open_file (dbm, argv[optind + 1])) < 0) {
		sprintf (error_msg, "dbunload: can't open file %s",
			argv[optind + 1]);
		db_error (error_msg);
		exit (1);
	}

	/*
	 * See if the user asked for the file to be scanned in some
	 * particular order.  If there is another argument, it is used
	 * as the name of the index to use reading the file.
	 */

	if (optind + 2 >= argc) {
		fieldnum = 1;
		dbm->d_files[filenum].f_current_index = 1;
		db_rewind (dbm, filenum);
	} else if ((fieldnum = db_open_index (dbm, filenum,
			argv[optind + 2])) < 0) {
		sprintf (error_msg, "dbunload: can't open index %s",
			argv[optind + 2]);
		db_error (error_msg);
		exit (1);
	}

	/*
	 * Scan the list of fields that were requested for the
	 * view.  Since we don't know the expected attributes of
	 * the view members, we can't use db_desc_view.
	 */

	if (view_fields) {

		/*
		 * Walk through the comma-separated list of fields.
		 * Each named fields will be looked up, and its
		 * identifier added to an array for later use.
		 */

		for (view_count = 0;view_fields;view_count++) {
			cp = strchr (view_fields, ',');
			if (cp)
				*cp++ = '\0';

			/*
			 * Get the field identifiers and put them in the
			 * list for db_view.
			 */

			if ((i = db_open_field (dbm, filenum,
						view_fields)) >= 0)
				view_list[view_count] = i;
			else {
				sprintf (error_msg, "dbunload: unknown field %s",
					view_fields);
				db_error (error_msg);
				exit (1);
			}
			view_fields = cp;
		}

		/* 
		 * Create a view with all of the fields that were found
		 */

		if (! db_view (dbm, filenum, view_count, view_list,
				view_packed)) {
			sprintf (error_msg, "dbunload: can't create file view");
			db_error (error_msg);
			exit (1);
		}

		/*
		 * Use db_read_view() to read the file
		 */

		db_read = db_read_view;
	} else

		/*
		 * Use db_read_record() to read the file
		 */

		db_read = db_read_record;

	/*
	 * Locate the file input buffer and get the record size.  A
	 * buffer will be allocated for converting the record to text
	 * format if the output isn't in binary format.
	 */

	if (view_count > 0) {
		record = dbm->d_files[filenum].f_current_view->v_buffer;
		size = dbm->d_files[filenum].f_current_view->v_size;
	} else {
		record = dbm->d_files[filenum].f_data_buffer;
		size = dbm->d_files[filenum].f_record_size;
	}
	if (! binary) {
		if (! (buf = malloc (2 * size))) {
			perror ("dbunload: out of memory");
			exit (1);
		}
	}

	/*
	 * Read the file a single record at a time.  The index which
	 * was selected earlier will be used, so the records will appear
	 * either in record or index order.
	 */

	while ((recno = (*db_read) (dbm, filenum, 0, IS_NEXT)) != -1) {
		if (binary) {

			/*
			 * Write the record out in binary format
			 */

			if (fwrite (record, size, 1, stdout) != size ||
					fflush (stdout)) {
				fprintf (stderr, "dbunload: write failed\n");
				exit (1);
			}
			continue;
		}

		/*
		 * Convert the record to text and output the string
		 */

		if (buf2string (&dbm->d_files[filenum], buf, delimiter, record)) {
			if (printf ("%s\n", buf) == EOF || fflush (stdout)) {
				fprintf (stderr, "dbunload: write failed\n");
				exit (1);
			}
		} else {
			fprintf (stderr,
				"dbunload: error converting record %ld\n",
					dbm->d_files[filenum].f_current_record);
			fflush (stderr);
		}
	}
	exit (0);
	/*NOTREACHED*/
}
