#!/bin/sh
#
#	Test 13: test DBSCHEMA -B option
#
#	@(#)test_13.sh	1.1	10:39:17	2/26/92
#
#	This test verifies that the -B option of DBSCHEMA produces
#	databases with the correct index block size.  It also
#	indirectly tests the isam_file_open() function with multiple
#	sized files in a database and the -v option to DBSHOW
#

DBPATH=. ; export DBPATH

rm -f test_13.sch test_13.out test_13.err test[0123].dat test[0123].idx
RC=1
set -e
trap 'rm -f test[0123].dat test[0123].idx test_13.sch test_13.out test_13.err ; exit $RC' 0 1 2 3 15

cat > test_13.sch << EOF
database test0

file test1

field field1 type int
field field2 type int

end
EOF
#
# create the database with 1K index blocks
#
./dbschema -B 1024 test_13.sch
./dbshow -v test0 > test_13.out
cmp -s - test_13.out << \EOF
database test0

{ block size = 512 }

file test1

{ record size = 8, block size = 1024 }

field	.test1	type	index unique
field	field1	type	int
field	field2	type	int

end
EOF
#
# now declare a second file with a 2K block size
#
cat > test_13.sch << EOF
database test0

file test2

field field1 type int
field field2 type int

end
EOF
#
# create the new file
./dbschema -B 2048 test_13.sch
./dbshow -v test0 > test_13.out
cmp -s - test_13.out << \EOF
database test0

{ block size = 512 }

file test1

{ record size = 8, block size = 1024 }

field	.test1	type	index unique
field	field1	type	int
field	field2	type	int

file test2

{ record size = 8, block size = 2048 }

field	.test2	type	index unique
field	field1	type	int
field	field2	type	int

end
EOF
#
# now declare a second file with a 4K block size
#
cat > test_13.sch << EOF
database test0

file test3

field field1 type int
field field2 type int

end
EOF
#
# create the new file
./dbschema -B 4096 test_13.sch
./dbshow -v test0 > test_13.out
cmp -s - test_13.out << \EOF
database test0

{ block size = 512 }

file test1

{ record size = 8, block size = 1024 }

field	.test1	type	index unique
field	field1	type	int
field	field2	type	int

file test2

{ record size = 8, block size = 2048 }

field	.test2	type	index unique
field	field1	type	int
field	field2	type	int

file test3

{ record size = 8, block size = 4096 }

field	.test3	type	index unique
field	field1	type	int
field	field2	type	int

end
EOF
#
# test for illegal block sizes.  must be power of 2 > 512 and <= MAXBLOCKSIZE
#
cat > test_13.sch << EOF
database test0

file test4

field field1 type int

end
EOF
#
# not a power of two
#
if ./dbschema -B 1234 test_13.sch > test_13.err 2>&1
then
	exit
fi
cmp -s - test_13.err << EOF
dbschema: invalid block size
EOF
#
# smaller than 512
#
if ./dbschema -B 128 test_13.sch > test_13.err 2>&1
then
	exit
fi
cmp -s - test_13.err << EOF
dbschema: invalid block size
EOF
#
# greater than 4096
#
if ./dbschema -B 8192 test_13.sch > test_13.err 2>&1
then
	exit
fi
cmp -s - test_13.err << EOF
dbschema: invalid block size
EOF
#
# Everything is OK
#
RC=0
